# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V61
0
1568
1569
1570
1600
1601
1611
1646
1648
1649
1652
1653
1748
1749
1750
1774
1776
1786
1789
1791
1792
1808
1809
1810
1840
1869
1920
2144
2155
2208
2221
2222
2229
2230
2248
68288
68294
68295
68296
68297
68299
68301
68322
68324
68325
68331
68336
68866
68867
68873
68874
68889
68890
68892
68893
68894
68895
68896
68897
68899
68900
END
